from datetime import datetime as dt
import logging
import os

from django.core.mail import EmailMessage
from botocore.exceptions import ClientError


from django.core.management.base import (BaseCommand, CommandParser,
                                         CommandError)

from openlxp_notifications.models import (email, recipient)

from openlxp_notifications.management.utils.ses_configuration import (
    overall_health)

logger = logging.getLogger('dict_config_logger')

TEMPLATE_ROOT = os.path.abspath(os.path.join(__file__, "../../.."))


def trigger_health_check():
    """Command to trigger email health check"""
    overall_health()


def trigger_update(email_type):
    """Command to trigger email notification"""

    trigger_health_check()

    # Replace sender@example.com with your "From" address.
    # This address must be verified with Amazon SES.
    SENDER = email_type.sender

    # Replace recipient@example.com with a "To" address. If your account
    # is still in the sandbox, this address must be verified.
    RECIPIENT = list(email_type.recipients.values_list("email_address",
                                                       flat=True))

    # The subject line for the email.
    SUBJECT = email_type.subject

    # The HTML body of the email.
    now = dt.now()
    datetimenow = now.strftime("%d/%m/%Y %H:%M:%S")
    for each_recipient in RECIPIENT:
        try:
            MESSAGE_HTML = email_type.template_type.message
            BODY_HTML = \
                open(TEMPLATE_ROOT +
                     '/templates/status_update_template.html').read()

            # Provide the contents of the email.
            recipient_obj = recipient.objects.get(
                email_address=each_recipient)

            name = (recipient_obj.first_name + " " +
                    recipient_obj.last_name)

            BODY_HTML = BODY_HTML.format(message=MESSAGE_HTML)
            BODY_HTML = BODY_HTML.format(name=name, datetime=datetimenow)

            mail = EmailMessage(SUBJECT, BODY_HTML, SENDER,
                                [each_recipient])
            mail.content_subtype = "html"
            mail.send()
        # Display an error if something goes wrong.
        except ClientError as e:
            logger.error(e.response['Error']['Message'])
            continue
        except Exception as e:
            logger.error(e)


class Command(BaseCommand):
    """Django command to send an emails to the filer/personas, when the log
    warning/error occurred in the metadata EVTVL process."""

    def add_arguments(self, parser: CommandParser) -> None:
        # parser.add_argument('email_references', nargs="+", type=str)
        parser.add_argument('email_references', type=str)
        return super().add_arguments(parser)

    def handle(self, *args, **options):
        """Email log notification is sent to filer/personas when warning/error
        occurred in EVTVL process"""
        # for email_reference in options['email_references']:
        try:
            email_type = email.objects.get(
                reference=options['email_references'])
        except email.DoesNotExist:
            raise CommandError('Email Reference "%s" does not exist' %
                               options['email_references'])

        trigger_update(email_type)



# Generated by Django 3.2.18 on 2024-05-30 16:46

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_notifications', '0003_emailconfiguration_banner'),
    ]

    operations = [
        migrations.CreateModel(
            name='email',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('sender', models.EmailField(blank=True, max_length=254, null=True)),
                ('reference', models.CharField(help_text='Enter email reference', max_length=200, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='recipient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(help_text='Enter recipient name', max_length=200)),
                ('last_name', models.CharField(help_text='Enter recipient name', max_length=200)),
                ('email_address', models.EmailField(help_text='Enter recipient Email ID', max_length=254, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='subject',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('subject', models.CharField(default='OpenLXP Conformance Alerts', help_text='Enter Subject for email', max_length=200)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='template',
            fields=[
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('template_id', models.BigAutoField(primary_key=True, serialize=False)),
                ('template_type', models.CharField(help_text='Enter Template Type', max_length=200, unique=True)),
                ('message', models.TextField(blank=True, help_text='Enter Email Message', null=True)),
                ('template_inputs', models.JSONField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.DeleteModel(
            name='EmailConfiguration',
        ),
        migrations.DeleteModel(
            name='ReceiverEmailConfiguration',
        ),
        migrations.DeleteModel(
            name='SenderEmailConfiguration',
        ),
        migrations.AddField(
            model_name='email',
            name='recipients',
            field=models.ManyToManyField(blank=True, related_name='email_recipients', to='openlxp_notifications.recipient'),
        ),
        migrations.AddField(
            model_name='email',
            name='subject',
            field=models.ForeignKey(blank=True, help_text='Select Email Subject', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='email_subject', to='openlxp_notifications.subject'),
        ),
        migrations.AddField(
            model_name='email',
            name='template_type',
            field=models.ForeignKey(blank=True, help_text='Select Email Template', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='email_template', to='openlxp_notifications.template'),
        ),
    ]



import logging
import os

from openlxp_notifications_project import settings

logger = logging.getLogger('dict_config_logger')

headers = {'Content-Type': 'application/json'}


"""
    Functions set up to extract environment values for
    AWS Simple Email Service (SES)

"""


def check_aws_access_key():
    """Extracts P1PS base endpoint"""
    aws_access_key = settings.AWS_ACCESS_KEY_ID

    if aws_access_key:
        logger.info("AWS Access key is present and set")
    else:
        logger.error("Team Token value is absent and not set")

    return aws_access_key


def check_aws_secret():
    """Extracts P1PS base endpoint"""
    aws_secret = settings.AWS_SECRET_ACCESS_KEY

    if aws_secret:
        logger.info("AWS Secret access Key  is present and set")
    else:
        logger.error("Team Token value is absent and not set")

    return aws_secret


def check_aws_region():
    """Extracts P1PS base endpoint"""
    aws_region = settings.AWS_DEFAULT_REGION

    if aws_region:
        logger.info("AWS Default region is present and set")
    else:
        logger.error("Team Token value is absent and not set")

    return aws_region


def overall_health():
    """Check the variable required for SES are set"""
    check_aws_access_key()
    check_aws_secret()
    check_aws_region()



import logging

import boto3

logger = logging.getLogger('dict_config_logger')


def email_verification(email):
    """Function to send email verification"""
    logger.info("Check if email id from user is verified")
    ses = boto3.client('ses')
    check = check_if_email_verified(email)

    if check:
        logger.error(str(email) + "Email is not verified")
        logger.info("Email is sent for Verification")
        try:
            response = ses.verify_email_identity(
                EmailAddress=email
            )
            logger.info(response)
        except Exception as e:
            logger.error(e)


def check_if_email_verified(email):
    """Function to check if email id from user is verified """
    list_emails = list_email_verified()
    if email in list_emails:
        logger.info("Email is already Verified")
        return False
    return True


def list_email_verified():
    """Function to return list of verified emails """

    ses = boto3.client('ses')
    try:
        response = ses.list_identities(
            IdentityType='EmailAddress',
            MaxItems=1000
        )
        logger.info(response['Identities'])
        return response['Identities']
    except Exception as e:
            logger.error(e)



from django.contrib import admin

from .models import (recipient, subject, template, email)


@admin.register(recipient)
class recipientAdmin(admin.ModelAdmin):
    list_display = ('first_name', 'last_name', 'email_address', )


@admin.register(subject)
class subjectAdmin(admin.ModelAdmin):
    list_display = ('subject', )


@admin.register(template)
class templateAdmin(admin.ModelAdmin):
    list_display = ('template_type', )
    # readonly_fields = ['template_type',]


@admin.register(email)
class emailAdmin(admin.ModelAdmin):
    list_display = ('reference', 'subject',
                    'template_type', )
    readonly_fields = ('created', 'modified', )



from django.db import models
from model_utils.models import TimeStampedModel

from openlxp_notifications.management.utils.ses_client import (
    email_verification)


class recipient(models.Model):
    """Model for POC Email Configuration """

    first_name = models.CharField(max_length=200,
                                  help_text='Enter recipient name',
                                  null=False, blank=False)
    last_name = models.CharField(max_length=200,
                                 help_text='Enter recipient name',
                                 null=False, blank=False)
    email_address = models.EmailField(
        max_length=254,
        help_text='Enter recipient Email ID',
        unique=True)

    def __str__(self):
        """String for representing the Model object."""
        return f'{self.email_address}'

    def save(self, *args, **kwargs):
        email_verification(self.email_address)
        return super(recipient, self).save(*args, **kwargs)


class subject(TimeStampedModel):
    """Model for Subject set up"""
    subject = models.CharField(max_length=200,
                               default='OpenLXP Conformance Alerts',
                               help_text="Enter Subject for email")

    def __str__(self):
        """String for representing the Model object."""
        return f'{self.subject}'


class template(TimeStampedModel):
    """Model for template set up"""
    template_id = models.BigAutoField(primary_key=True)
    template_type = models.CharField(max_length=200,
                                     help_text='Enter Template Type',
                                     unique=True, null=False, blank=False)
    message = models.TextField(help_text="Enter Email Message",
                               blank=True, null=True,)
    template_inputs = models.JSONField(blank=True, null=True)

    def __str__(self):
        """String for representing the Model object."""
        return f'{self.template_type}'


class email(TimeStampedModel):
    """Model for email configuration"""
    sender = models.EmailField(blank=True, null=True)
    reference = models.CharField(max_length=200,
                                 help_text='Enter email reference',
                                 unique=True, null=False, blank=False)
    subject = models.ForeignKey(subject, related_name='email_subject',
                                on_delete=models.SET_NULL, blank=True,
                                null=True,
                                help_text="Select Email Subject")
    template_type = models.ForeignKey(template, related_name='email_template',
                                      on_delete=models.SET_NULL,
                                      blank=True, null=True,
                                      help_text="Select Email Template")
    recipients = models.ManyToManyField(recipient,
                                        related_name='email_recipients',
                                        blank=True)



# Generated by Django 3.2.3 on 2022-06-13 15:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_notifications', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='emailconfiguration',
            old_name='Logs_Type',
            new_name='Content_Type',
        ),
    ]
